@@ -305,8 +305,7 @@ check_send_occ_msg_dowork(struct context *c)
             const struct key_type *kt = &c->c1.ks.key_type;
 
             /* OCC message have comp/fragment headers but not ethernet headers */
-            payload_hdr = frame_calculate_payload_overhead(&c->c2.frame, &c->options,
-                                                           kt, false);
+            payload_hdr = frame_calculate_payload_overhead(0, &c->options, kt);
 
             /* Since we do not know the payload size we just pass 0 as size here */
             proto_hdr = frame_calculate_protocol_header_size(kt, &c->options, false);
