@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2013-2022 Heiko Hund <heiko.hund@sophos.com>
+ *  Copyright (C) 2013-2023 Heiko Hund <heiko.hund@sophos.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -24,6 +24,11 @@
 #ifndef _SERVICE_H
 #define _SERVICE_H
 
+/* We do not support non-unicode builds */
+#ifndef UNICODE
+#define UNICODE
+#endif
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #elif defined(_MSC_VER)
@@ -48,7 +53,6 @@
 #define M_ERR     (MSG_FLAGS_ERROR)                    /* error */
 
 typedef enum {
-    automatic,
     interactive,
     _service_max
 } openvpn_service_type;
@@ -72,14 +76,9 @@ typedef struct {
     BOOL append;
 } settings_t;
 
-extern openvpn_service_t automatic_service;
 extern openvpn_service_t interactive_service;
 extern LPCTSTR service_instance;
 
-VOID WINAPI ServiceStartAutomaticOwn(DWORD argc, LPTSTR *argv);
-
-VOID WINAPI ServiceStartAutomatic(DWORD argc, LPTSTR *argv);
-
 VOID WINAPI ServiceStartInteractiveOwn(DWORD argc, LPTSTR *argv);
 
 VOID WINAPI ServiceStartInteractive(DWORD argc, LPTSTR *argv);
